window.Gleam.initSearch([{"doc":"effect","title":"effect","content":"# effect\n\n[![Package Version](https://img.shields.io/hexpm/v/effect)](https://hex.pm/packages/effect)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/effect/)\n\n```sh\ngleam add effect@1.2.0\n```\n\nThis library includes an implementation of `Effect` that handles results differently.\nSee [effect_result](#effect-with-result) for details.\n\n```gleam\nimport effect\n\npub fn main() {\n  let google: Effect(Result(String, Error)) = {\n    use uri &lt;- effect.try_replace_error(\n      uri.parse(&quot;https://www.google.com&quot;),\n      UriParse,\n    )\n    use req &lt;- effect.try_replace_error(request.from_uri(uri), UriParse)\n    use res &lt;- effect.try_await_map_error(fetch.send(req), Fetch)\n    use text &lt;- effect.try_await_map_error(fetch.read_text_body(res), Fetch)\n\n    text.body |&gt; Ok |&gt; effect.dispatch\n  }\n\n  use res &lt;- effect.perform(google)\n  case res {\n    Ok(body) -&gt; io.println(body)\n    Error(e) -&gt; e |&gt; string.inspect |&gt; io.println_error\n  }\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/effect&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\n```\n\n# Effect With Result\n\nA lightweight library for modeling asynchronous effects and error handling in Gleam, inspired by [effect-ts](https://github.com/Effect-TS/core). \n\nThis library allows you to treat asynchronous operations and potential failures as **first-class effectful computations**, which can be composed in a purely functional style before finally being executed.\n\nThe motivation was to create an API for dealing with promises from [gleam_promises](https://hexdocs.pm/gleam_javascript/gleam/javascript/promise.html) without having to &quot;color&quot; functions.\n\n## Table of Contents\n\n1. [Overview](#overview)  \n2. [Usage](#usage)  \n   - [Basic Construction](#basic-construction)  \n   - [Chaining Effects](#chaining-effects)  \n   - [Handling Promises](#handling-promises)  \n\n## Overview\n\n- **Asynchronous &amp; Error-Encoded**: Each `Effect(a, e)` can produce either an `Ok(a)` (success) or an `Error(e)` (failure).  \n- **Composable**: Build up complex workflows using `map`, `map_error`, and `try`.  \n- **Lifts Promises**: Bridge JavaScript `promise.Promise(Result(a, e))` into the Gleam effect world with `from_promise`, `try_await`, and `try_await_map_error`.  \n- **Perform**: Nothing executes until you call `perform(effect, callback)`. This separates definition from execution.\n\n## Usage\n\nBelow are common usage scenarios demonstrating how to create and compose effects, handle failures, work with promises, and finally execute the effect.\n\n### Basic Construction\n\n```gleam\nimport effect/effect_result as effect\nimport gleam/io\n\npub fn main() {\n  // Construct an effect from a plain Result:\n  let eff_from_result = effect.from_result(Ok(&quot;Hello&quot;))\n\n  // Succeed / Fail shortcuts:\n  let eff_ok = effect.succeed(42)\n  let eff_err = effect.fail(&quot;Oops!&quot;)\n\n  // Perform them:\n  effect.perform(eff_from_result, fn(res) {\n    case res {\n      Ok(str) -&gt; io.println(&quot;Got: &quot; &lt;&gt; str)\n      Error(err) -&gt; io.println(&quot;Error: &quot; &lt;&gt; err)\n    }\n  })\n}\n```\n\n### Chaining Effects\n\nUse `try` to sequence operations, only continuing on success, or short-circuiting on error:\n\n```gleam\nimport effect/effect_result as effect\nimport gleam/io\n\ntype TooSmallError {\n  TooSmallError\n}\n\npub fn main() {\n  let computation =\n    effect.succeed(10)\n    // successful effect\n    |&gt; effect.try(fn(x: Int) {\n      // \n      case x &lt; 5 {\n        True -&gt; effect.fail(TooSmallError)\n        False -&gt; effect.succeed(x * 2)\n      }\n    })\n    |&gt; effect.map(fn(double: Int) { double + 1 })\n\n  effect.perform(computation, fn(res: Result(Int, TooSmallError)) {\n    case res {\n      // prints 21 if x=10\n      Ok(n) -&gt; io.println(&quot;Final result: &quot; &lt;&gt; int.to_string(n))\n      // TooSmallError\n      Error(_err) -&gt; io.println(&quot;Error: &quot; &lt;&gt; &quot;Too small!&quot;)\n    }\n  })\n}\n```\n\n### Handling Promises\n\nYou can convert a `promise.Promise(Result(a, e))` into an `Effect(a, e)` using `from_promise`. Then compose it with `try_await` to sequence a follow-up effect:\n\n```gleam\ntype FetchError {\n  FetchError(String)\n}\n\n\nfn fetch_data() -&gt; promise.Promise(Result(String, FetchError)) {\n  // For example, some JavaScript network call\n  promise.new(fn(resolve) {\n    // resolve(Ok(&quot;Server data&quot;))\n    resolve(Error(FetchError(&quot;Network error&quot;)))\n  })\n}\n\nfn main() {\n  // `try_await` waits for Ok(a), then calls `f(a) -&gt; Effect(b, e)`\n  let eff =\n    effect.try_await(fetch_data(), fn(data) {\n      // do something with the data\n      effect.from_result(Ok(&quot;Got data: &quot; &lt;&gt; data))\n    })\n\n  effect.perform(eff, fn(res) {\n    case res {\n      // Server Data\n      Ok(msg) -&gt; io.println(msg)\n      // Network Error\n      Error(FetchError(msg)) -&gt; io.println(&quot;Failed: &quot; &lt;&gt; msg)\n    }\n  })\n}\n```\n\nIf you need to map the error type *before* continuing, use `try_await_map_error`.\n","url":"index.html"},{"doc":"effect","title":"Effect","content":"pub opaque type Effect(msg)\n The `Effect` type represents a description of side effects as data. Each\n effect specifies:\n 1. The operations to perform\n 2. The type of messages that will be sent back to your program\n\n\nSynonyms:\neffect.Effect\neffect Effect","url":"effect.html#Effect"},{"doc":"effect","title":"dispatch","content":"pub fn dispatch(value: a) -&gt; Effect(a)\n Convert a value into an effect that will dispatch that value when performed.\n\n \n value\n |&gt; dispatch\n |&gt; perform(handle_value)\n \nSynonyms:\neffect.dispatch\neffect dispatch","url":"effect.html#dispatch"},{"doc":"effect","title":"from","content":"pub fn from(effect: fn(fn(a) -&gt; Nil) -&gt; Nil) -&gt; Effect(a)\n Create a custom effect from a function that takes a dispatch callback.\n The dispatch callback can be used to send messages back to your program.\n\n \n from(fn(dispatch) {\n   dispatch(MyMessage)\n })\n \nSynonyms:\neffect.from\neffect from","url":"effect.html#from"},{"doc":"effect","title":"map","content":"pub fn map(effect: Effect(a), f: fn(a) -&gt; b) -&gt; Effect(b)\n Transform the messages produced by an effect. This is useful when you need\n to adapt effects from one part of your program to work with another.\n\n \n effect\n |&gt; map(fn(msg) { TransformedMessage(msg) })\n \nSynonyms:\neffect.map\neffect map","url":"effect.html#map"},{"doc":"effect","title":"map_result","content":"pub fn map_result(\n  effect: Effect(Result(a, b)),\n  f: fn(a) -&gt; Effect(Result(c, b)),\n) -&gt; Effect(Result(c, b))\n Transform an effect containing a Result by providing a function that operates on\n the success value and returns another Result-containing effect. This is useful\n for chaining effects where each step can potentially fail. Errors from either\n the input effect or the transformation function will short-circuit the chain.\n\n \n use response &lt;- map_result(initial_effect)\n handle_response(response)\n \nSynonyms:\neffect.map_result\neffect map_result","url":"effect.html#map_result"},{"doc":"effect","title":"none","content":"pub fn none() -&gt; Effect(a)\n Create an effect that does nothing. This is useful when you need to return\n an effect but don&#39;t actually want to perform any operations.\n\nSynonyms:\neffect.none\neffect none","url":"effect.html#none"},{"doc":"effect","title":"param","content":"pub fn param(f: fn(a) -&gt; b) -&gt; fn(a) -&gt; b\n Helper function similar to clip&#39;s parameter function. Provides an alternative\n syntax for building curried functions. The following are equivalent:\n\n \n fn(a) { fn(b) { thing(a, b) } }\n\n {\n   use a &lt;- param\n   use b &lt;- param\n   thing(a, b)\n }\n \n\n Mostly used internally.\n\nSynonyms:\neffect.param\neffect param","url":"effect.html#param"},{"doc":"effect","title":"perform","content":"pub fn perform(effect: Effect(a), dispatch: fn(a) -&gt; b) -&gt; Nil\n Run an effect by providing a dispatch function that will receive any\n messages produced by the effect.\n\n \n effect\n |&gt; perform(fn(msg) {\n   case msg {\n     Ok(data) -&gt; handle_success(data)\n     Error(e) -&gt; handle_error(e)\n   }\n })\n \nSynonyms:\neffect.perform\neffect perform","url":"effect.html#perform"},{"doc":"effect","title":"try","content":"pub fn try(\n  res: Result(a, b),\n  f: fn(a) -&gt; Effect(Result(c, b)),\n) -&gt; Effect(Result(c, b))\n Handle a Result by providing a function that produces an effect for the\n success case. Errors are automatically converted into effects.\n\n \n use data &lt;- try(parse_data())\n process_data(data)\n \nSynonyms:\neffect.try\neffect try","url":"effect.html#try"},{"doc":"effect","title":"try_await","content":"pub fn try_await(\n  pres: Promise(Result(a, b)),\n  f: fn(a) -&gt; Effect(Result(c, b)),\n) -&gt; Effect(Result(c, b))\n Handle a Promise containing a Result by providing a function that produces\n an effect for the success case. This is particularly useful for handling\n async operations like HTTP requests.\n\n \n use response &lt;- try_await(fetch.send(request))\n process_response(response)\n \nSynonyms:\neffect.try_await\neffect try_await","url":"effect.html#try_await"},{"doc":"effect","title":"try_await_map_error","content":"pub fn try_await_map_error(\n  pres: Promise(Result(a, b)),\n  map_error: fn(b) -&gt; c,\n  f: fn(a) -&gt; Effect(Result(d, c)),\n) -&gt; Effect(Result(d, c))\n Similar to `try_await` but allows mapping error values before they&#39;re\n dispatched. This is commonly used when you want to wrap external errors\n in your own error type.\n\n \n use response &lt;- try_await_map_error(\n   fetch.send(request),\n   fn(e) { NetworkError(e) }\n )\n process_response(response)\n \nSynonyms:\neffect.try_await_map_error\neffect try_await_map_error","url":"effect.html#try_await_map_error"},{"doc":"effect","title":"try_await_replace_error","content":"pub fn try_await_replace_error(\n  pres: Promise(Result(a, b)),\n  e: c,\n  f: fn(a) -&gt; Effect(Result(d, c)),\n) -&gt; Effect(Result(d, c))\n Similar to `try_await` but allows replacing error values before they&#39;re\n dispatched. This is commonly used when you want to wrap external errors\n in your own error type.\n\n \n use response &lt;- try_await_replace_error(\n   fetch.send(request),\n   NetworkError,\n )\n process_response(response)\n \nSynonyms:\neffect.try_await_replace_error\neffect try_await_replace_error","url":"effect.html#try_await_replace_error"},{"doc":"effect","title":"try_map_error","content":"pub fn try_map_error(\n  res: Result(a, b),\n  map_error: fn(b) -&gt; c,\n  f: fn(a) -&gt; Effect(Result(d, c)),\n) -&gt; Effect(Result(d, c))\n Similar to `try` but allows mapping error values before they&#39;re dispatched.\n You can emulate this using `try` and `result.map_error`.\n\n \n use response &lt;- try_map_error(\n   fetch.send(request),\n   fn(e) { NetworkError(e) }\n )\n process_response(response)\n \nSynonyms:\neffect.try_map_error\neffect try_map_error","url":"effect.html#try_map_error"},{"doc":"effect","title":"try_replace_error","content":"pub fn try_replace_error(\n  res: Result(a, b),\n  e: c,\n  f: fn(a) -&gt; Effect(Result(d, c)),\n) -&gt; Effect(Result(d, c))\n Similar to `try` but allows replacing error values before they&#39;re dispatched.\n You can emulate this using `try` and `result.replace_error`.\n\n \n use response &lt;- try_replace_error(\n   fetch.send(request),\n   NetworkError,\n )\n process_response(response)\n \nSynonyms:\neffect.try_replace_error\neffect try_replace_error","url":"effect.html#try_replace_error"},{"doc":"effect","title":"effect","content":"","url":"effect.html"},{"doc":"effect/effect_result","title":"Actions","content":"pub type Actions(a, e) {\n  Actions(dispatch: fn(Result(a, e)) -&gt; Nil)\n}\n Holds the `dispatch` function used internally by each effect step.\n Typically you won&#39;t create `Actions` directly; it&#39;s used by `perform`.\nActions(dispatch: fn(Result(a, e)) -&gt; Nil)\n\n\nSynonyms:\neffect/effect_result.Actions\neffect/effect_result Actions","url":"effect/effect_result.html#Actions"},{"doc":"effect/effect_result","title":"Effect","content":"pub opaque type Effect(a, e)\n An effect that can produce either an Ok(a) or an Error(e).\n Use `perform` to execute an Effect, providing a callback that\n receives the final Result(a, e).\n\nSynonyms:\neffect/effect_result.Effect\neffect/effect_result Effect","url":"effect/effect_result.html#Effect"},{"doc":"effect/effect_result","title":"fail","content":"pub fn fail(err: a) -&gt; Effect(b, a)\n Create an Effect that immediately fails with error `err`.\n Shorthand for `from_result(Error(err))`.\nSynonyms:\neffect/effect_result.fail\neffect/effect_result fail","url":"effect/effect_result.html#fail"},{"doc":"effect/effect_result","title":"from","content":"pub fn from(\n  effect: fn(fn(Result(a, b)) -&gt; Nil) -&gt; Nil,\n) -&gt; Effect(a, b)\n Create an Effect from a callback that manually dispatches Ok or Error.\n Example:\n \n from(fn(dispatch) {\n   dispatch(Ok(42))\n })\n \nSynonyms:\neffect/effect_result.from\neffect/effect_result from","url":"effect/effect_result.html#from"},{"doc":"effect/effect_result","title":"from_option","content":"pub fn from_option(opt: Option(a), if_none: b) -&gt; Effect(a, b)\n Create an Effect from an option, mapping the None case into the error channel\n Useful for wrapping an option into the effect system when None is considered an error.\nSynonyms:\neffect/effect_result.from_option\neffect/effect_result from_option","url":"effect/effect_result.html#from_option"},{"doc":"effect/effect_result","title":"from_promise","content":"pub fn from_promise(pres: Promise(Result(a, b))) -&gt; Effect(a, b)\n Convert a JavaScript Promise&lt;Result(a, e)&gt; into an Effect(a, e).\n The Effect will dispatch either Ok(a) or Error(e) based on the\n resolved promise value.\nSynonyms:\neffect/effect_result.from_promise\neffect/effect_result from_promise","url":"effect/effect_result.html#from_promise"},{"doc":"effect/effect_result","title":"from_result","content":"pub fn from_result(result: Result(a, b)) -&gt; Effect(a, b)\n Create an Effect that immediately dispatches a pre-existing Result(a, e).\n Useful for wrapping a plain result in the effect system.\nSynonyms:\neffect/effect_result.from_result\neffect/effect_result from_result","url":"effect/effect_result.html#from_result"},{"doc":"effect/effect_result","title":"map","content":"pub fn map(effect: Effect(a, b), f: fn(a) -&gt; c) -&gt; Effect(c, b)\n Transform the success value in an Effect from type `a` to `b`.\n The error type `e` is left unchanged.\n Example:\n \n succeed(2)\n |&gt; map(fn(x) { x * 2 })  // Ok(4)\n \nSynonyms:\neffect/effect_result.map\neffect/effect_result map","url":"effect/effect_result.html#map"},{"doc":"effect/effect_result","title":"map_error","content":"pub fn map_error(\n  effect: Effect(a, b),\n  f: fn(b) -&gt; c,\n) -&gt; Effect(a, c)\n Transform the error value in an Effect from type `e` to `e2`.\n The success value `a` is left unchanged.\n Example:\n \n fail(&quot;Oops!&quot;)\n |&gt; map_error(fn(e) { &quot;Mapped: &quot; &lt;&gt; e })\n \nSynonyms:\neffect/effect_result.map_error\neffect/effect_result map_error","url":"effect/effect_result.html#map_error"},{"doc":"effect/effect_result","title":"perform","content":"pub fn perform(\n  effect: Effect(a, b),\n  callback: fn(Result(a, b)) -&gt; Nil,\n) -&gt; Nil\n Execute an Effect by supplying the final callback to handle\n the Result(a, e). Use this to “run” the effect.\n Example:\n \n let eff = succeed(10)\n perform(eff, fn(res) {\n   case res {\n     Ok(value) -&gt; io.println(&quot;Got: \\(value)&quot;)\n     Error(e)  -&gt; io.println(&quot;Error: \\(e)&quot;)\n   }\n })\n \nSynonyms:\neffect/effect_result.perform\neffect/effect_result perform","url":"effect/effect_result.html#perform"},{"doc":"effect/effect_result","title":"succeed","content":"pub fn succeed(x: a) -&gt; Effect(a, b)\n Create an Effect that immediately succeeds with value `x`.\n Shorthand for `from_result(Ok(x))`.\nSynonyms:\neffect/effect_result.succeed\neffect/effect_result succeed","url":"effect/effect_result.html#succeed"},{"doc":"effect/effect_result","title":"try","content":"pub fn try(\n  effect: Effect(a, b),\n  f: fn(a) -&gt; Effect(c, b),\n) -&gt; Effect(c, b)\n Chain two Effects. If `effect` succeeds with Ok(a), call `f(a)` to\n produce a new Effect(b, e). If `effect` fails, propagate the error.\n Example:\n \n succeed(2)\n |&gt; try(fn(x) {\n   if x &gt; 0 { succeed(x * 10) }\n   else { fail(&quot;Can&#39;t multiply&quot;) }\n })\n \nSynonyms:\neffect/effect_result.try\neffect/effect_result try","url":"effect/effect_result.html#try"},{"doc":"effect/effect_result","title":"try_await","content":"pub fn try_await(\n  pres: Promise(Result(a, b)),\n  f: fn(a) -&gt; Effect(c, b),\n) -&gt; Effect(c, b)\n Wait on `pres` (a Promise&lt;Result(a, e)&gt;) and then,\n if it succeeds with Ok(a), run `f(a)`.\n Otherwise, dispatch Error(e).\nSynonyms:\neffect/effect_result.try_await\neffect/effect_result try_await","url":"effect/effect_result.html#try_await"},{"doc":"effect/effect_result","title":"try_await_map_error","content":"pub fn try_await_map_error(\n  pres: Promise(Result(a, b)),\n  f_err: fn(b) -&gt; c,\n  f: fn(a) -&gt; Effect(d, c),\n) -&gt; Effect(d, c)\n Like `try_await`, but first map the error using `f_err`.\n If the promise fails with Error(e), transform it into Error(e2),\n then call `f` if successful.\nSynonyms:\neffect/effect_result.try_await_map_error\neffect/effect_result try_await_map_error","url":"effect/effect_result.html#try_await_map_error"},{"doc":"effect/effect_result","title":"effect/effect_result","content":"","url":"effect/effect_result.html"}]);